//-----------------------------------------------------------------------------
// Source File Name: PubPanel.java
//
// Description: Controls and lays out the publication panel on the main window
//              
// Licensed Materials - Property of IBM
//
// 5648-C63
// (C) Copyright IBM Corp. 2002, 2003 All Rights Reserved.
//
// US Government Users Restricted Rights - Use, duplication or
// disclosure restricted by GSA ADP Schedule Contract with
// IBM Corp.
//
// Version %Z% %W% %I% %E% %U%
//
//-----------------------------------------------------------------------------
package com.ibm.MQIsdp.utility;

import java.awt.*;
import java.awt.event.*;

import javax.swing.*;
import javax.swing.border.*;

public class PubPanel implements ActionListener {
	private JPanel pubPanel;
	private JComboBox topic;
	private Integer[]  qos = { new Integer(0), new Integer(1), new Integer(2) };
	private JComboBox qosList;
	private JButton pubButton;
    private JButton hexButton;
    private boolean hexDisplay = false;
	private JTextArea pubData;
	private JCheckBox retained;
	private MQIsdpFrame mqisdpMgr = null;
	private JLabel pubLabel = null;
	private static final String PANEL_TITLE = " Publish Messages";
		

	public PubPanel( JPanel thePubPanel, MQIsdpFrame aMqisdpMgr ) {
		pubPanel = thePubPanel;
        pubPanel.setBorder( new EtchedBorder() );

        mqisdpMgr = aMqisdpMgr;
        
        init();
        
	}	
	
	public void init() {
		pubPanel.setLayout(new BorderLayout() );
        
		topic = new JComboBox();
		topic.setEditable( true );
        topic.setMaximumSize(MQIsdpFrame.TEXT_FIELD_DIMENSION);
        topic.setMaximumRowCount(5);
		
        qosList = new JComboBox( qos );
		qosList.setSelectedIndex( 0 );
		qosList.setMaximumSize( MQIsdpFrame.DROP_DOWN_DIMENSION );
				
		pubData = new JTextArea(3,30);
		pubData.setBorder( new BevelBorder( BevelBorder.LOWERED ) );
        pubData.setMargin( MQIsdpFrame.TEXT_MARGINS );
								
        retained = new JCheckBox();
        retained.setSelected( false );

        // Create the components to go in the NORTH panel
        // Create the title label for the pubPanel
        pubLabel = new JLabel( PANEL_TITLE  + " - text display" );
        Font f = pubLabel.getFont();
        pubLabel.setFont( new Font( f.getName(), Font.BOLD, f.getSize() + 1 ) );

        // To add a title to the panel place the Topic label, topic text field, qos combo box and retained check box in a horizontal layout
        // Then place this horizontal layout below the title "Publish Messages"
        JPanel topicBox = new JPanel();
        topicBox.setLayout( new BoxLayout( topicBox, BoxLayout.X_AXIS ) );
        topicBox.add( new JLabel(" Topic:") );
        topicBox.add( topic );
        topicBox.add( new JLabel(" QoS:") );
        topicBox.add( qosList );
        topicBox.add( new JLabel(" Retained:") );
        topicBox.add( retained );
        
        // Add everything to the panel that will be inserted into the NORTH of the pubPanel
        JPanel titleAndTopic = new JPanel();
        titleAndTopic.setLayout( new GridLayout(2,1) );
        titleAndTopic.add( pubLabel );
        titleAndTopic.add( topicBox );

        // Create the components to go in the EAST panel
        // Add the button to a FlowLayout to stop it resizing
        hexDisplay = false;
        hexButton = new JButton( "Hex" );
        hexButton.addActionListener( this );
        
		pubButton = new JButton( "Publish" );
		pubButton.addActionListener( this );

        JPanel buttons = new JPanel();
        buttons.setLayout( new GridLayout(3,1) );
        
        buttons.add( pubButton );
        buttons.add( new JLabel("") );
        buttons.add( hexButton );
        		
		JPanel buttonLayout = new JPanel();
		buttonLayout.add( buttons );

        // Now add the title and topic options, data text area and the publish button to the pubPanel
        pubPanel.add( titleAndTopic, BorderLayout.NORTH );
        pubPanel.add( new JScrollPane(pubData), BorderLayout.CENTER );
        pubPanel.add( buttonLayout, BorderLayout.EAST );
	}	

    public boolean updateTopicList( String topicName ) {
        return mqisdpMgr.updateComboBoxList( topic, topicName );
    }
    	    		
    public void actionPerformed( ActionEvent e ) {
    	
    	if ( e.getActionCommand().equals("Publish") ) {
    		if ( hexDisplay == true ) {
    			// Convert the display to the characters we want to send
    			toCharString();
    		}	
        	String pubText = pubData.getText();
        	String topicName = (String)topic.getSelectedItem();
    	
    	    if ( updateTopicList( topicName ) ) {
    		    // If we needed to update this list, then update the subscriber panel's topic list
        		mqisdpMgr.updateSubscribeTopicList( topicName );
        	}
            mqisdpMgr.publish( topicName, pubText.getBytes(), qosList.getSelectedIndex(), retained.isSelected() );
    	} else {
    		if ( hexDisplay == false ) {
        		toHexString();
    		} else {
        		toCharString();
        	}	
    	}	
    }		
    
    private void toHexString() {
    	String pubText = pubData.getText();
    	StringBuffer hexText = new StringBuffer();
    	
    	byte[] pubBytes = pubText.getBytes();

    	for( int i=0; i<pubBytes.length; i++ ) {
    		int byteValue = pubBytes[i];

            // Change the byte value from a signed to unsigned value
            // e.g. A byte of value 0xAA is treated as -86 and displayed incorrectly as 0xFFFFFFAA
            // Adding 256 to this value changes it to 170 which is displayed correctly as 0xAA
    		if (byteValue < 0) {
    			byteValue += 256;
    		}	

    		if ( byteValue < 16 ) {
        		hexText.append( "0" + Integer.toHexString(byteValue) );
    		} else {
        		hexText.append( Integer.toHexString(byteValue) );
    		}		
    	}	
    	
        hexDisplay = true;
        pubLabel.setText( PANEL_TITLE + " - hexadecimal display" );
        mqisdpMgr.setTitleText("");
        hexButton.setText("Text");
    	pubData.setText( hexText.toString() );
    }	
    
    private void toCharString() {
    	String hexText = pubData.getText();
    	
    	if ( hexText.length() % 2 != 0 ) {
    		mqisdpMgr.setTitleText( "Odd number of hex characters!" );
    	} else {
    		try {
                byte[] charArray = new byte[hexText.length()/2];    
        		for( int i=0; i<charArray.length; i++ ) {
                    // Take each pair of bytes from the input and convert to a character			
                    // Use the Integer parseInt method to take a 2 byte string in base 16 and turn it into an integer.
    	    		charArray[i] = (byte)Integer.parseInt( hexText.substring( i*2,(i*2)+2), 16 );
    	    		//  The code below doesn't seem to work as expected
    	    		// The parseByte method will not accept A-F chars in a base 16 pair of bytes???
    	    		// charArray[i] = Byte.parseByte( hexText.substring( i*2,(i*2)+2), 16 );
    		    }	

                hexDisplay = false;
                pubLabel.setText( PANEL_TITLE + " - text display" );
                mqisdpMgr.setTitleText("");
                hexButton.setText("Hex");
    		    pubData.setText( new String(charArray) );
    		} catch( NumberFormatException nfe ) {
    			mqisdpMgr.setTitleText( "Invalid hexadecimal data!" );
    		}	    
    	}	
    }	

}
